/*
 *  Copyright (C) 2021 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <drivers/i2c.h>
#include <kernel/dpl/DebugP.h>
#include "ti_drivers_config.h"
#include "ti_drivers_open_close.h"
#include "ti_board_open_close.h"
#include <drivers/i2c.h>
#define I2C_ADDRESS_MIN 0x03
#define I2C_ADDRESS_MAX 0x77

static void i2c_read_error_handler(int32_t status);

void detect_all_i2c_devices(I2C_Handle i2cHandle) {
    int32_t stat;
    uint8_t deviceAddress;

    for (deviceAddress = I2C_ADDRESS_MIN; deviceAddress <= I2C_ADDRESS_MAX; deviceAddress++) {
        stat = I2C_probe(i2cHandle, deviceAddress);
        if (stat == SystemP_SUCCESS) {
            DebugP_log("[I2C] Device found at address 0x%02x \r\n", deviceAddress);
        }
    }
}

void empty_main(void *arg0)
{
    int32_t         status = SystemP_SUCCESS;
    DebugP_log("\nstatus: %c", status);
    I2C_Handle      i2cHandle;
    uint8_t buffer[2] = {0};

    /* STEP 1:
     *
     * Opening drivers, I2C being one of them.
     */

    /* STEP 2:
     *
     * Initializes I2C Handle.
     */
    i2cHandle = gI2cHandle[CONFIG_I2C0];
    
    // Detect all I2C devices
    detect_all_i2c_devices(i2cHandle);
    
   
    buffer[0] = 0x0D;
    buffer[1] = 0xBF;

    I2C_Transaction i2cTransaction;
    I2C_Transaction_init(&i2cTransaction);
    i2cTransaction.slaveAddress = 0x22;
    i2cTransaction.writeBuf = buffer;
    i2cTransaction.writeCount = 2;
    status = I2C_transfer(i2cHandle, &i2cTransaction);
    i2c_read_error_handler(status);
    DebugP_assert(SystemP_SUCCESS == status);
    DebugP_log("\n%d", status);

}

static void i2c_read_error_handler(int32_t status)
{
    switch(status)
    {
        case I2C_STS_ERR:
            DebugP_logError("\nGeneric error occurred");
            break;
        case I2C_STS_ERR_TIMEOUT:
            DebugP_logError("\nTimeout error occurred");
            break;
        case I2C_STS_ERR_NO_ACK:
            DebugP_logError("\nNo acknowledgement received\n");
            break;
        case I2C_STS_ERR_ARBITRATION_LOST:
            DebugP_logError("\nSample Arbitration lost\n");
            break;
        case I2C_STS_ERR_ACCESS_ERROR:
            DebugP_logError("\nBus Access error occurred\n");
            break;
    }

    return;
}
